import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {MatExpansionModule, MatExpansionPanel} from "@angular/material/expansion";
import {DataSpinnerComponent} from "@component/data-spinner/data-spinner.component";
import {MatChipListboxChange, MatChipsModule} from '@angular/material/chips';
import {FormControl, ReactiveFormsModule} from "@angular/forms";
import {AsyncPipe} from "@angular/common";
import {map, Observable, of} from "rxjs";
import {FacultyResponse} from "@model/facultyResponse";
import {GroupResponse} from "@model/groupResponse";

@Component({
  selector: 'app-schedule-tabs-group',
  standalone: true,
  imports: [
    MatExpansionModule,
    DataSpinnerComponent,
    MatChipsModule,
    ReactiveFormsModule,
    AsyncPipe
  ],
  templateUrl: './schedule-tabs-group.component.html',
  styleUrl: './schedule-tabs-group.component.css'
})
export class ScheduleTabsGroupComponent {
  protected facultiesId: number | null = null;
  protected courseNumber: number | null = null;
  protected filteredGroups: Observable<GroupResponse[]> = of([]);
  protected courseNumbers: Observable<number[]> = of([]);
  protected groups: Observable<GroupResponse[]> = of([]);

  protected chipCourse: FormControl = new FormControl();
  protected chipGroup: FormControl = new FormControl();

  @ViewChild('courseNumberPanel') courseNumberPanel!: MatExpansionPanel;
  @ViewChild('groupPanel') groupPanel!: MatExpansionPanel;

  @Input() faculties: Observable<FacultyResponse[]> = of([]);

  @Input() set groupsSet(data: Observable<GroupResponse[]>) {
    this.groups = data;
    this.courseNumbers = this.groups.pipe(
      map(data => data.map(g => g.courseNumber)),
      map(courseNumbersArray => courseNumbersArray.filter((value, index, self) => self.indexOf(value) === index)),
      map(uniqueCourseNumbers => uniqueCourseNumbers.sort((a, b) => a - b))
    );
  }

  @Output() groupSelected = new EventEmitter<number>();
  @Output() facultySelected = new EventEmitter<number>();

  protected selectedFaculty(event: MatChipListboxChange) {
    this.courseNumber = null;
    this.groups = of([]);
    this.chipGroup.reset();
    this.chipCourse.reset();

    if (event.value === undefined || event.value === null) {
      this.facultiesId = null;
      return;
    }
    this.facultiesId = event.value;
    this.courseNumberPanel.open();

    this.facultySelected.emit(this.facultiesId!);
  }

  protected selectCourseNumber(event: MatChipListboxChange) {
    this.filteredGroups = of([]);
    this.chipGroup.reset();

    if (event.value === undefined || event.value === null) {
      this.courseNumber = null;
      return;
    }

    this.courseNumber = event.value;
    this.groupPanel.open();
    this.groups.subscribe(data =>
      this.filteredGroups = of(data.filter(g => g.courseNumber === this.courseNumber)));
  }

  protected selectGroup(event: MatChipListboxChange) {
    if (event.value === undefined || event.value === null)
      return;

    this.groupPanel.close();
    this.groupSelected.emit(event.value);
  }
}
